/* fits.c - several useful(?) functions
 *
 * $Id: fits.c,v 1.17 2006/06/02 04:07:38 mik Exp $
 *
 * $Log: fits.c,v $
 * Revision 1.17  2006/06/02 04:07:38  mik
 * "parasite" symbols in comment field...
 *
 * Revision 1.16  2006/02/11 17:35:41  mik
 * some functions were removed... fitshdr is not affected
 *
 * Revision 1.15  2005/08/25 16:35:44  mik
 * ...cosmetics...
 *
 * Revision 1.14  2005/08/25 16:24:11  mik
 * ...error in one new function... fitshdr is not affected.
 *
 * Revision 1.13  2005/08/25 10:14:27  mik
 * several function were added (are not used in fitshdr)...
 *
 * Revision 1.12  2005/07/26 15:31:01  mik
 * ... '-' is legal symbol in key names...
 *
 * Revision 1.11  2005/01/21 18:17:19  mik
 * ver.1.0.4 rel.1: spaces in comments (in tail) were removed...
 *
 * Revision 1.10  2004/11/24 17:55:28  mik
 * ver.1.0.3: undefined values (stupid standard specification!) are supported...
 *
 * Revision 1.9  2004/10/27 14:35:56  mik
 * ...lint...
 *
 * Revision 1.8  2004/09/22 12:04:25  mik
 * ...again, around "SIMPLE = F"...
 *
 * Revision 1.7  2004/09/22 12:00:58  mik
 * ..."SIMPLE = F" fitshdr_getattr() problem...
 *
 * Revision 1.6  2004/09/16 15:14:08  mik
 * ...comments related, again...
 *
 * Revision 1.5  2004/09/16 12:49:54  mik
 * ...error in handling comment for STRING types variables...
 *
 * Revision 1.4  2004/09/15 13:16:32  mik
 * ...some bugs were fixed...
 *
 * Revision 1.3  2004/08/25 13:59:36  mik
 * MEF & stdio/pipe related bugs were fixed...
 *
 * Revision 1.2  2004/08/24 20:53:41  mik
 * 1st beta version...
 *
 * Revision 1.1  2004/08/20 12:28:23  mik
 * semi-working version...
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <unistd.h>
#include "fits.h"


int
sread_fits_key(const char *s, FITS_KEY *key){
  char *cptr, *chr, *str;
  int	i, is_done;
  long   lval;
  double dval;
  char *lptr, *dptr;
  
  if(!s || !key) return(-1);

  key->type = FITS_UNDEF;
  key->has_value = 0;
  key->is_defined = 0;
  
  strncpy(key->card, s, FITS_CARD_LEN);
  (key->card)[FITS_CARD_LEN]='\0';

  strncpy(key->name, s, 8) ; key->name[8] = '\0';
  /* it'll be better check for "freestyle" key-card format */

  str = key->card;

  cptr = str + 8;
  for(chr=key->name; *chr; ++chr){
    if(isalpha(*chr) && isupper(*chr)) continue;
    if(*chr == '-' && chr != key->name) continue;
    if(isdigit(*chr) && chr!=key->name) continue;
    if(isspace(*chr) || *chr == '='){ /* violation of standard */
      cptr = str + (chr-(key->name));
      *chr = '\0';
      break;
    }
    *(key->name) = '\0';
    cptr = str;
    break;
  }

  for(;*cptr;++cptr) if(*cptr != ' ') break;

  if( *(key->name) && !strncmp(cptr, "= ", 2)){ /* check for KEYs value */
    key->has_value = 1;
    cptr += 2;
    /* this is not completely correct for STRINGs */
    for(;*cptr;++cptr) if(*cptr != ' ') break;
    if(*cptr == ' ' || !*cptr || *cptr == '/'){ /* undefined value */
      key->type = FITS_UNDEF;
      key->is_defined = 0;
/*
      key->comment[0] = '\0';
      key->is_defined = 0;
      key->card_position = cptr;
      return(1);
 */
    }
    else if( *cptr == '\''){
      key->type = FITS_SVAL; /* string */
      chr = cptr;
      for(++cptr, is_done=0;*cptr;++cptr){
        if(*cptr == '\''){
          if(is_done){
            is_done = 0; /* means double quote, not delimiter */
            continue;
          }
          else{
            is_done = 1;
          }
        }
        else if(is_done){
          break; /* means -- string delimiter */
        }
      }
      if(is_done){
        if(cptr-chr>2){
          strncpy(key->value.sval, chr+1, cptr-chr-2);
          (key->value.sval)[cptr-chr-2] = '\0';
        }
        else{
          *(key->value.sval)='\0';
        }
/*
        if(*cptr) ++cptr;
 */
        key->is_defined = 1;
      }
      else{
        key->is_defined = 0;
        key->card_position = cptr;
        return(4); /* unclosed ' */
      }
    }
    else{
      if( *cptr == 'T' || *cptr == 'F'){
        key->type = FITS_BVAL; /* bool */
        key->value.bval = (*cptr == 'T')?1:0;
        if(*cptr) ++cptr;
        key->is_defined = 1;
      }
      else{
        if((chr=strchr(cptr, '/'))) *chr = '\0';
	lval = strtol(cptr, &lptr, 10);
	dval = strtod(cptr, &dptr);
        if(chr) *chr = '/'; /* restore previous value */

        if(key->type == FITS_IVAL){
          key->value.ival = lval;
          cptr = lptr;
          key->is_defined = 1;
        }
        else if(key->type == FITS_FVAL){
          key->value.fval = dval;
          cptr = dptr;
          key->is_defined = 1;
        }
        else{
          if(lptr==cptr && dptr==cptr){
            key->is_defined = 0;
            key->type = FITS_UNDEF;
            key->card_position = cptr;
            return(5); /* is not number */
          }
          else if(lptr == dptr){
            key->type = FITS_IVAL;
            key->value.ival = lval;
            cptr = lptr;
            key->is_defined = 1;
          }
          else if( dptr<lptr ){ /* strange result but... */
            key->type = FITS_IVAL;
            key->value.ival = lval;
            cptr = lptr;
            key->is_defined = 1;
          }
          else if( dptr>lptr ){
            key->type = FITS_FVAL;
            key->value.fval = dval;
            cptr = dptr;
            key->is_defined = 1;
          }
        }
      }
    }
  }
  else{
    key->type = FITS_NOVAL;
  }

  
  key->comment[0] = '\0';

  for(;*cptr;++cptr) if(*cptr != ' ') break;

  if(*cptr && key->type != FITS_NOVAL){
    if(*cptr == '/'){
      ++cptr;
    }
    else{
      key->comment[0] = '\0';
      key->card_position = cptr;
      return(3); /* unknown character instead of comment delimiter */
    }
  }

  for(;*cptr;++cptr) if(*cptr != ' ') break;

  if((i=FITS_CARD_LEN-(cptr-str)) > 0 && *cptr != ' '){
    strncpy(key->comment, cptr, i);
    key->comment[FITS_CARD_LEN] = '\0';
    key->comment[i] = '\0';
  }
  else{
    key->comment[0] = '\0';
  }

  key->card_position = cptr;
  if((i=strlen(key->comment))){
    for(cptr=(key->comment) + i-1; cptr >= key->comment ; --cptr){
      if(*cptr != ' ') break;
      *cptr = '\0';
    }
  }
  return(0);
}

static int
is_ok_naxis(FITSHDR_ATTR *hdr_attr){
  int	i;

  if(hdr_attr->naxis == INT_MIN) return(0);
  if(!(hdr_attr->naxis)) return(1);

  if((hdr_attr->naxis) < 0) return(0);
  if(!(hdr_attr->naxisn)) return(0);
  for(i=0; i<hdr_attr->naxis; ++i){
    if((hdr_attr->naxisn)[i] == INT_MIN) return(0);
/*
    if((hdr_attr->naxisn)[i] < 0) return(0);
 */
  }

  return(2);
}

static int
is_naxis_key(char *s){
  long l;
  char *cptr;

  if(!s) return(0);

  if(!strncmp(s, "NAXIS", sizeof("NAXIS")-1)){
    l = strtol(s+sizeof("NAXIS")-1, &cptr, 10);
    if(cptr != s+sizeof("NAXIS")-1 && !*cptr && l > 0 && l < 1000) return((int)l);
  }
  return(0);
}


int
fitshdr_getattr(const char raw_hdr[FITS_BLK_SIZE], FITSHDR_ATTR *hdr_attr){
  char	*cptr;
  int	i, rc, n, ii;
  static FITS_KEY	fkey;

  for(cptr=(char *)raw_hdr, i=0; i<FITS_BLK_SIZE/FITS_CARD_LEN; cptr += FITS_CARD_LEN, ++i){
    if((rc=sread_fits_key(cptr, &fkey))<0) return(-64-rc);

    if(!strncmp(fkey.name, "SIMPLE", sizeof("SIMPLE"))){
      if(!fkey.is_defined || fkey.type != FITS_BVAL) return(-16);
      if(fkey.value.bval)
        hdr_attr->type |= HTYPE_PRIMARY;
      else
        hdr_attr->type |= HTYPE_UNKNOWN;
      continue;
    }


    if(!strncmp(fkey.name, "GROUPS", sizeof("GROUPS"))){
      if(!fkey.is_defined || fkey.type != FITS_BVAL) return(-18);
      if(fkey.value.bval) hdr_attr->type |= HTYPE_RGROUP;
      continue;
    }


    if(!strncmp(fkey.name, "XTENSION", sizeof("XTENSION"))){
      if(!fkey.is_defined || fkey.type != FITS_SVAL) return(-17);
      if(!(fkey.value.bval)) return(17);

      hdr_attr->type |= HTYPE_EXT;
      if(!strncmp(fkey.value.sval, "IMAGE", sizeof("IMAGE"))){
        hdr_attr->type |= HTYPE_IMAGE;
      }
      else if(!strncmp(fkey.value.sval, "BINTABLE", sizeof("BINTABLE"))){
        hdr_attr->type |= HTYPE_BTABLE;
      }
      else if(!strncmp(fkey.value.sval, "TABLE", sizeof("TABLE"))){
        hdr_attr->type |= HTYPE_ATABLE;
      }
      else{
      }
      continue;
    }



    if(!strncmp(fkey.name, "BITPIX", sizeof("BITPIX"))){
      if(!fkey.is_defined || fkey.type != FITS_IVAL) return(-1);
      if(hdr_attr->bitpix != INT_MIN) return(1); /* double declaration */
      hdr_attr->bitpix = fkey.value.ival;
      continue;
    }
    if(!strncmp(fkey.name, "NAXIS", sizeof("NAXIS"))){
      if(!fkey.is_defined || fkey.type != FITS_IVAL) return(-2);
      if(hdr_attr->naxisn) return(2);
      if(fkey.value.ival > 0 && !((hdr_attr->naxisn)=(int *)malloc(sizeof(int)*fkey.value.ival))) return(2);
      for(ii=0; ii<fkey.value.ival; ++ii) (hdr_attr->naxisn)[ii] = INT_MIN;
      hdr_attr->naxis = fkey.value.ival;
      continue;
    }

/* NAXISn */
    if((n=is_naxis_key(fkey.name))){
      if(!fkey.is_defined || fkey.type != FITS_IVAL) return(-3);
      if(n <= 0 || n > hdr_attr->naxis || !(hdr_attr->naxisn)) return(3);
      if((hdr_attr->naxisn)[n-1] != INT_MIN) return(3);
      (hdr_attr->naxisn)[n-1] = fkey.value.ival;
      continue;
    }

    if(!strncmp(fkey.name, "PCOUNT", sizeof("PCOUNT"))){
      if(!fkey.is_defined || fkey.type != FITS_IVAL) return(-4);
      if(hdr_attr->pcount != LONG_MIN || fkey.value.ival < 0) return(4);
      hdr_attr->pcount = fkey.value.ival;
      continue;
    }

    if(!strncmp(fkey.name, "GCOUNT", sizeof("GCOUNT"))){
      if(!fkey.is_defined || fkey.type != FITS_IVAL) return(-5);
      if(hdr_attr->gcount != INT_MIN || fkey.value.ival < 0) return(5);
      hdr_attr->gcount = fkey.value.ival;
      continue;
    }

    if(!strncmp(fkey.name, "END", sizeof("END"))){
      hdr_attr->is_completed = 1;
      hdr_attr->end_off = cptr-raw_hdr;
      fitshdr_attr_fill(hdr_attr);
      return(0);
    }
  }
  return(0);
}

FITSHDR_ATTR *
fitshdr_attr_alloc(){
  FITSHDR_ATTR	*attr;
  if(!(attr=malloc(sizeof(FITSHDR_ATTR)))) return(NULL);
  fitshdr_attr_init(attr);
  return(attr);
}

void
fitshdr_attr_free(FITSHDR_ATTR *attr){
  fitshdr_attr_reset(attr);
  free(attr);
}

void
fitshdr_attr_init(FITSHDR_ATTR *hdr_attr){
  hdr_attr->naxisn = NULL;
  hdr_attr->is_completed = 0;
  hdr_attr->naxis   = INT_MIN;
  hdr_attr->bitpix  = INT_MIN;
  hdr_attr->datalen = INT_MIN;
  hdr_attr->pcount  = LONG_MIN;
  hdr_attr->gcount  = INT_MIN;
  hdr_attr->type    = HTYPE_UNDEF;
  hdr_attr->hdr_len = 0;
  hdr_attr->hdr_off = LONG_MIN;
  hdr_attr->end_off = LONG_MIN;
}

void
fitshdr_attr_reset(FITSHDR_ATTR *hdr_attr){
  if(hdr_attr->naxisn) free(hdr_attr->naxisn);
  fitshdr_attr_init(hdr_attr);
}

int
fitshdr_attr_fill(FITSHDR_ATTR *hdr_attr){
  long	l, b;
  int	i;

  if(!is_ok_naxis(hdr_attr)){
    return(-8);
  }
  if(hdr_attr->naxis == 0){
    hdr_attr->datalen = 0;
    return(0);
  }
  if(hdr_attr->bitpix == INT_MIN) return(-1);
  if((hdr_attr->bitpix)%8 || !(hdr_attr->bitpix)) return(1); /* BITPIX = 8,16,32,... */

  if((b=(hdr_attr->bitpix)/8) < 0) b = -b;

  if(((hdr_attr->type)&HTYPE_RGROUP)){ /* random group */
    if(hdr_attr->naxis < 1 || (hdr_attr->naxisn)[0]) return(2);
    if(hdr_attr->gcount < 1 || hdr_attr->pcount < 0) return(2);

    for(l=1, i=1; i<hdr_attr->naxis; ++i) l *= (hdr_attr->naxisn)[i];
    if(l<=0) return(3);

    l += hdr_attr->pcount;
    hdr_attr->datalen = l*b*hdr_attr->gcount;

    return(0);
  }

  for(l=1, i=0; i<hdr_attr->naxis; ++i) l *= (hdr_attr->naxisn)[i];
  if(l<=0) return(3);

  if(hdr_attr->pcount != LONG_MIN){
    if(hdr_attr->pcount < 0) return(5);
    l += hdr_attr->pcount;
  }

  if(hdr_attr->gcount != INT_MIN){
    if(hdr_attr->gcount <= 0) return(6);
    l *= hdr_attr->gcount;
  }

  if(l <= 0) return(7);
  
  hdr_attr->datalen = l * b;

  return(0);
}
