#ifndef __FITS_H__
#define __FITS_H__  1

#define FITS_BLK_SIZE (2880)
#define FITS_CARD_LEN (80)

typedef enum {
  FITS_UNDEF=0,		/* undefined */
  FITS_NOVAL, 		/* comment? */
  FITS_SVAL, 		/* string */
  FITS_BVAL, 		/* Boolean */
  FITS_IVAL, 		/* int */
  FITS_FVAL,		/* float */
  FITS_CIVAL,		/* complex integer */
  FITS_CFVAL,		/* complex float */
  FITS_CVAL		/* complex */
} FITS_KEY_TYPES;

typedef struct fits_key{
  int	has_value;
  int   is_defined;
  FITS_KEY_TYPES	type;
  char  name[8+1];
  union {
    char        sval[FITS_CARD_LEN+1];
    long        ival;
    int         bval;
    double      fval;
  } value;
  char  comment[FITS_CARD_LEN+1];
  char  card[FITS_CARD_LEN+1];
  char  *card_position;
} FITS_KEY;

#define	HTYPE_UNDEF	(0)
#define HTYPE_PRIMARY	(0x1)
#define HTYPE_EXT	(0x2)
#define HTYPE_IMAGE	(0x4)
#define HTYPE_RGROUP	(0x8)
#define HTYPE_ATABLE	(0x10)
#define HTYPE_BTABLE	(0x20)
#define HTYPE_VTABLE	(0x40)	/* variable length */
#define HTYPE_UNKNOWN	(0x80)

typedef struct fitshdr_attr{
  int	is_completed;
  int	naxis;
  int	*naxisn;
  long	pcount;
  int	gcount;
  int	bitpix;
  long	datalen;
  int	type;
  long  hdr_len;
  long  hdr_off;
  long  end_off;
  void	*reserved;	/* generic pointer to associate external info */
} FITSHDR_ATTR;

int sread_fits_key(const char *str, FITS_KEY *keyID);
int fitshdr_getattr(const char raw_hdr[FITS_BLK_SIZE], FITSHDR_ATTR *hdr_attr);

FITSHDR_ATTR *fitshdr_attr_alloc(void);
void fitshdr_attr_free(FITSHDR_ATTR *attr);

void fitshdr_attr_reset(FITSHDR_ATTR *hdr_attr);
void fitshdr_attr_init(FITSHDR_ATTR *hdr_attr);
int fitshdr_attr_fill(FITSHDR_ATTR *hdr_attr);
#endif
