#!/bin/sh

# fitshdr_edit.sh - script, which illustrates how to use 'fitshdr' utility
#                   to edit (interactively) existent FITS-headers.
#
# $Id: fitshdr_edit.sh,v 1.16 2005/07/21 16:55:40 mik Exp $


usage(){
  echo "Usage: `basename ${0}` [options] file.fits"
  echo "  --help                  - shows this message"
  echo "  --editor=EDITOR         - use editor EDITOR [${editor}]"
  echo "  --nobackups             - do not create backup file"
  echo "  --version               - display version information"
}

editor="${EDITOR:-vi}"


IS_NOBACKUPS="NO"
while [ "${#}" -ge "1" ] ; do
  case "${1}" in
        "--help")
		usage;
		exit 0;
	;;
        "--")
		shift;
		break;
        ;;
        "--nobackups")
		IS_NOBACKUPS="YES";
        ;;
        "--version")
                TMP_VAR=`echo '$Revision: 1.16 $' | sed -n "s/^.*Revision: \([0-9][.0-9]*\).*/\1/p"`
		echo "`basename ${0}`: version ${TMP_VAR}"
		exit
        ;;
        "--editor="*)
                TMP_VAR=`echo ${1} | sed -n "s/^--editor=\([^ ].*\) *$/\1/p"`
                if [ "${TMP_VAR}" = "" ] ; then
                   echo "`basename ${0}`: \"${1}\": bad name."
                   exit 1;
                fi
                EDITOR="${TMP_VAR}"
        ;;
        "-"*)
                echo "`basename ${0}`: \"${1}\": bad option!"
                usage;
                exit 1;
        ;;
        *)
		break;
        ;;
  esac
  shift;
done
if [ "${#}" -gt 1 ] ; then
   echo "`basename ${0}`: \"${2}\": bad option."
   exit 1;
fi

if [ "${#}" -lt 1 ] ; then
   echo "`basename ${0}`: FITS-file name is missed!"
   usage;
   exit 1;
fi

[ "${EDITOR}" = "" ] && EDITOR="${editor}"


tmpdir="${TMPDIR:-/tmp}"
FILE="${1}"
NEWFILE="${FILE}.$$"
HDRFILE="${tmpdir}/tmp$$.hdr"
HDRLINES="${tmpdir}/tmp$$.hdr.lines"
LOGFILE="${tmpdir}/tmp$$.log"

HDRLINENO="1"

if [ ! -f "${FILE}" ] ; then
   echo "`basename ${0}`: FITS-file \"${FILE}\" does not exist!"
   exit 10
fi

if [ ! -w "${FILE}" ] ; then
   echo "`basename ${0}`: FITS-file \"${FILE}\" is not writable!"
   exit 11
fi

fitshdr --li --em "${FILE}" > "${HDRFILE}"
TMP_VAR="${?}"
if [ "${TMP_VAR}" != "0" ] ; then
   echo "`basename ${0}`: some problems with FITS-file headers (or 'fitshdr')!?"
   rm -f "${HDRFILE}"
   exit ${TMP_VAR};
fi

cat "${HDRFILE}" | sed -n "s/^ *\([0-9][0-9]*:\).*/\1/p" > "${HDRLINES}"
if [ "${?}" != "0" ] ; then
   echo "`basename ${0}`: can't create file with header offsets!"
   rm -f "${HDRFILE}"
   rm -f "${HDRLINES}"
   exit 12
fi

while true ; do
  ${EDITOR} +"${HDRLINENO}" "${HDRFILE}"

  fitshdr --li --em ${FILE} | diff >/dev/null 2>&1 - "${HDRFILE}"

  if [ "${?}" = "0" ] ; then
    rm -f "${HDRFILE}" "${LOGFILE}" "${HDRLINES}" >/dev/null 2>&1
    [ "${IS_NOBACKUPS}" != "YES" -a -f "${NEWFILE}" ] && rm -f "${NEWFILE}"
    echo "`basename ${0}`: FITS-file \"${FILE}\" was not changed."
    exit 0
  fi

  TMP_VAR=`cat "${HDRFILE}" | expand | \
           sed -n "s/^ *\([0-9][0-9]*:\).*/\1/p" | \
           diff -c - "${HDRLINES}" | sed -n "1,/^[!-+] /p" | \
           tail -2 | head -1 | sed "s/^ *//g"`

  if [ "${TMP_VAR}" != "" ] ; then
    echo "`basename ${0}`: WARNING!!! ACHTUNG!!! ATTENTION!!!"
    echo "Offset(s) for some header's line(s) was(were) changed or removed!"
    echo "As result, all removed lines, for instance, will NOT be affected, simply!!!!"
    echo "See fitshdr_edit(1) man-page, NOTES section."
    echo -n "Do You want to continue processing anyway? (yes/no/quit): "
    read ANS
    if [ "${ANS}" = "yes" -o "${ANS}" = "y" -o "${ANS}" = "YES" -o "${ANS}" = "Yes" -o "${ANS}" = "Y" ] ; then
       true
    else
       if [ "${ANS}" = "quit" -o "${ANS}" = "q" -o "${ANS}" = "QUIT" -o "${ANS}" = "Quit" -o "${ANS}" = "Q" ] ; then
         rm -f "${HDRFILE}" "${NEWFILE}" "${LOGFILE}" "${HDRLINES}" >/dev/null 2>&1
         exit 5
       fi
       HDRLINENO=`cat "${HDRFILE}" | expand | sed -n "/^ *${TMP_VAR}/="`
       HDRLINENO="${HDRLINENO:-1}"
       continue
    fi
  fi


  if [ "${IS_NOBACKUPS}" != "YES" ] ; then
    cp -p "${FILE}" "${NEWFILE}" >/dev/null 2>&1

    if [ "${?}" != "0" ] ; then
      rm -f "${HDRFILE}" "${NEWFILE}" "${LOGFILE}" "${HDRLINES}" >/dev/null 2>&1
      echo "`basename ${0}`: can't create backup file!"
      exit 2
    fi

    fitshdr -pv -H "${HDRFILE}" --li "${NEWFILE}" 2>"${LOGFILE}"
  else
    fitshdr -p -n -H "${HDRFILE}" --li "${FILE}" 2>"${LOGFILE}" && fitshdr -pv -H "${HDRFILE}" --li "${FILE}" 2>"${LOGFILE}"
  fi

  if [ "${?}" != "0" ] ; then
    echo "`basename ${0}`: some problems with new FITS-headers?!"
    tail -8 < "${LOGFILE}"
    HDRLINENO=`cat ${LOGFILE} | grep "${HDRFILE}:[1-9][0-9]*:" | tail -1 | sed -n "s#^fitshdr: ${HDRFILE}:\([1-9][0-9]*\):.*#\1#p"`
    HDRLINENO="${HDRLINENO:-1}"

    echo "" ; echo -n "Do You want to continue editing? (yes/no): "
    read ANS
    if [ "${ANS}" = "yes" -o "${ANS}" = "y" -o "${ANS}" = "YES" -o "${ANS}" = "Yes" -o "${ANS}" = "Y" ] ; then
       continue
    fi

    rm -f "${HDRFILE}" "${NEWFILE}" "${LOGFILE}" "${HDRLINES}" >/dev/null 2>&1
    echo "`basename ${0}`: editing was aborted!"
    exit 3
  else
    [ -s "${LOGFILE}" ] && tail -10 "${LOGFILE}"
    break
  fi
done

rm -f "${HDRFILE}" "${LOGFILE}" "${HDRLINES}" >/dev/null 2>&1
if [ "${IS_NOBACKUPS}" != "YES" ] ; then
  mv -f "${FILE}" "${FILE}~" && mv -f "${NEWFILE}" "${FILE}"
else
  true
fi

if [ "${?}" != 0 ] ; then
  echo "`basename ${0}`: some problems with renaming/copying!"
  rm -f "${NEWFILE}"
  exit 4
fi
