#!/bin/sh

# fitshdr_expand.sh - script, which illustrates how to use 'fitshdr'
#                     utility to expand/extend existent FITS-header.
#
# $Id: fitshdr_expand.sh,v 1.6 2004/11/16 13:48:20 mik Exp $

usage(){
  echo "Usage: `basename ${0}` [options] file.fits"
  echo "  --help                - shows this message"
  echo "  --header-no=NN        - expand header No.NN [${headerno}]"
  echo "  --add-blocks=BB       - add BB blocks [${addblocks}]"
  echo "  --nobackups           - do not create backup file"
  echo "  --version             - display version information"
}

headerno="0"
addblocks="1"
IS_NOBACKUPS="NO"
while [ "${#}" -ge "1" ] ; do
  case "${1}" in
        "--help")
		usage;
		exit 0;
	;;
        "--")
		shift;
		break;
        ;;
        "--header-no="*)
                TMP_VAR=`echo ${1} | sed -n "s/^--header-no= *\([0-9][0-9]*\) *$/\1/p"`
                if [ "${TMP_VAR}" = "" ] ; then
                   echo "`basename ${0}`: \"${1}\": bad argument."
                   exit 1;
                fi
                HEADERNO="${TMP_VAR}"
        ;;
        "--nobackups")
		IS_NOBACKUPS="YES";
        ;;
        "--version")
                TMP_VAR=`echo '$Revision: 1.6 $' | sed -n "s/^.*Revision: \([0-9][.0-9]*\).*/\1/p"`
		echo "`basename ${0}`: version ${TMP_VAR}"
		exit
        ;;
        "--add-blocks="*)
                TMP_VAR=`echo ${1} | sed -n "s/^--add-blocks= *\([1-9][0-9]*\) *$/\1/p"`
                if [ "${TMP_VAR}" = "" ] ; then
                   echo "`basename ${0}`: \"${1}\": bad argument."
                   exit 1;
                fi
                ADDBLOCKS="${TMP_VAR}"
        ;;
        "-"*)
                echo "`basename ${0}`: \"${1}\": bad option!"
                usage;
                exit 1;
        ;;
        *)
		break;
        ;;
  esac
  shift;
done

if [ "${#}" -gt 1 ] ; then
   echo "`basename ${0}`: \"${2}\": bad option."
   exit 1;
fi

if [ "${#}" -lt 1 ] ; then
   echo "`basename ${0}`: FITS-file name is missed!"
   usage;
   exit 1;
fi

[ "${ADDBLOCKS}" = "" ] && ADDBLOCKS="${addblocks}"
[ "${HEADERNO}" = "" ] && HEADERNO="${headerno}"


FILE="${1}"

tmpdir="${TMPDIR:-/tmp}"
NEWFILE="${FILE}.$$"
HDRFILE="${tmpdir}/tmp$$.hdr"

if [ ! -f "${FILE}" ] ; then
   echo "`basename ${0}`: FITS-file \"${FILE}\" does not exist!"
   exit 10
fi

if [ ! -w "${FILE}" ] ; then
   echo "`basename ${0}`: FITS-file \"${FILE}\" is not writable!"
   exit 11
fi

fitshdr --line-offsets --empty-lines --header-offsets "${FILE}" > "${HDRFILE}"
TMP_VAR="${?}"
if [ "${TMP_VAR}" != "0" ] ; then
   echo "`basename ${0}`: some problems with FITS-file headers (or 'fitshdr')!?"
   rm -f "${HDRFILE}"
   exit ${TMP_VAR};
fi

LASTHDRNO=`grep "^;;; [0-9][0-9]*: HEADER No\.[0-9][0-9]*$" < "${HDRFILE}" | wc -l | sed -n "s/^[^0-9]*\([1-9][0-9]*\)[^0-9]*$/\1/p"`
if [ "${LASTHDRNO}" = "" ] ; then
   echo "`basename ${0}`: ${FILE}: can't find any valid FITS-header!"
   rm -f "${HDRFILE}"
   exit 20
fi

# "-ge", because LASTHDRNO the headers' COUNTER, not last header's number!
if [ "${HEADERNO}" -ge "${LASTHDRNO}" ] ; then
   echo "`basename ${0}`: ${FILE}: header to expand (no.${HEADERNO}) does not exist!"
   rm -f "${HDRFILE}"
   exit 21
fi

#HDRNO=`expr ${HEADERNO} - 1`
HDRNO="${HEADERNO}"
mv -f "${HDRFILE}" "${HDRFILE}~" && \
sed -n "/^;;; [0-9][0-9]*: HEADER No.${HDRNO}.*/,/^[0-9][0-9]*:END$/p" < "${HDRFILE}~" > "${HDRFILE}" && \
rm -f "${HDRFILE}~"

if [ "$?" != "0" ] ; then
   echo "`basename ${0}`: some problems to write in `dirname ${HDRFILE}`/!"
   rm -f "${HDRFILE}~" "${HDRFILE}"
   exit 22
fi

HDRBLK0=`sed < "${HDRFILE}" -n "s/^;;; [0-9][0-9]*: BLOCK No.\([0-9][0-9]*\)$/\1/p" | head -1`
# echo "DEBUG: HDRBLK0=$HDRBLK0"

if [ "${HDRBLK0}" = "" ] ; then
   echo "`basename ${0}`: ${FILE}: header no.${HEADERNO}: can't find 1st block number!"
   rm -f "${HDRFILE}"
   exit 23
fi

HDRBLKS=`grep  < "${HDRFILE}" "^;;; [0-9][0-9]*: BLOCK No.[0-9][0-9]*$" | wc -l | sed -n "s/^[^0-9]*\([1-9][0-9]*\)[^0-9]*$/\1/p"`
# echo "DEBUG: HDRBLKS=$HDRBLKS"

if [ "${HDRBLKS}" = "" ] ; then
   echo "`basename ${0}`: ${FILE}: header no.${HEADERNO}: can't find total blocks number!"
   rm -f "${HDRFILE}"
   exit 24
fi

dd bs=2880 if="${FILE}" count="${HDRBLK0}" of="${NEWFILE}" 2>/dev/null
if [ "${?}" != "0" ] ; then
   echo "`basename ${0}`: 'dd' error! Permission denied, disk space!?"
   rm -f "${HDRFILE}" "${NEWFILE}"
   exit 29
fi

( sed < "${HDRFILE}" "/^[0-9][0-9]*:END$/d"
  BLKADD=`expr "${ADDBLOCKS}" - 1`
  while [ "${BLKADD}" -gt "0" ] ; do
    BLKADD=`expr "${BLKADD}" - 1`
    echo "+2880:"
  done
  echo "+2880:END"
) | fitshdr --line-offsets --discard-old-headers --edit --file="${NEWFILE}"
if [ "${?}" != "0" ] ; then
   echo "`basename ${0}`: 'fitshdr' ERROR! File '${FILE}' was not changed!"
   rm -f "${HDRFILE}" "${NEWFILE}"
   exit 30
fi

HDRBLK0=`expr "${HDRBLK0}" + "${HDRBLKS}"`
HDRBLKS=`expr "${HDRBLK0}" + "${ADDBLOCKS}"`
dd if="${FILE}" bs=2880 skip="${HDRBLK0}" seek="${HDRBLKS}" of="${NEWFILE}" 2>/dev/null
if [ "${?}" != "0" ] ; then
   echo "`basename ${0}`: 'dd' error! Permission denied, disk space!?"
   rm -f "${HDRFILE}" "${NEWFILE}"
   exit 31
fi

fitshdr --no-line-offsets --no-empty-lines --no-header-offsets --no-block-offsets --no-data-offsets "${FILE}" > "${HDRFILE}"
TMP_VAR="${?}"
if [ "${TMP_VAR}" != "0" ] ; then
   echo "`basename ${0}`: some problems with FITS-file headers (or 'fitshdr')!?"
   rm -f "${HDRFILE}" "${NEWFILE}"
   exit ${TMP_VAR};
fi

fitshdr --no-line-offsets --no-empty-lines --no-header-offsets --no-block-offsets --no-data-offsets "${NEWFILE}" | \
diff - "${HDRFILE}" >/dev/null 2>&1
if [ "${?}" != "0" ] ; then
   echo "`basename ${0}`: differences between old and new headers were found!"
   echo "`basename ${0}`: operation was aborted!"
   rm -f "${HDRFILE}" "${NEWFILE}"
   exit 51
fi

mv -f "${FILE}" "${FILE}~" && mv -f "${NEWFILE}" "${FILE}"
TMP_VAR="${?}"

if [ "${IS_NOBACKUPS}" = "YES" ] ; then
  if [ "${TMP_VAR}" != "0" ] ; then
     echo "`basename ${0}`: ALARM! Can't move/rename files!"
     echo "`basename ${0}`: Disk space? Permission denied? Something else?"
     echo "`basename ${0}`: To avoid lossing information, check ${FILE} and ${FILE}~!"
     rm -f "${HDRFILE}" "${NEWFILE}"
     exit 61
  fi
  rm -f "${FILE}~" >/dev/null
else
  if [ "${TMP_VAR}" != "0" ] ; then
     echo "`basename ${0}`: ALARM! Can't create backup file!"
     echo "`basename ${0}`: Disk space? Permission denied? Something else?"
     echo "`basename ${0}`: To avoid lossing information, check ${FILE} and ${FILE}~!"
     rm -f "${HDRFILE}" "${NEWFILE}"
     exit 62
  fi
fi

rm -f "${HDRFILE}"
